@@ -12,6 +12,7 @@ import {
     TupleStorageFactoryService,
     VortexService,
     VortexStatusService,
+    TupleStorageBatchSaveArguments,
 } from "@synerty/vortexjs";
 import { ChunkedIndexEncodedChunkTuple } from "./ChunkedIndexEncodedChunkTuple";
 import { ChunkedIndexUpdateDateTuple } from "./ChunkedIndexUpdateDateTuple";
@@ -26,7 +27,7 @@ import {
     chunkedIndexTuplePrefix,
 } from "../PluginNames";
 import {
-    DeviceOfflineCacheControllerService,
+    DeviceOfflineCacheService,
     OfflineCacheStatusTuple,
 } from "@peek/peek_core_device";
 
@@ -121,7 +122,12 @@ function keyChunk(modelSetKey: string, key: string): string {
 @Injectable()
 export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
     private UPDATE_CHUNK_FETCH_SIZE = 5;
-    private OFFLINE_CHECK_PERIOD_MS = 15 * 60 * 1000; // 15 minutes
+
+    // Every 100 chunks from the server
+    private SAVE_POINT_ITERATIONS = 100;
+
+    // Saving the cache after each chunk is so expensive, we only do it every 20 or so
+    private chunksSavedSinceLastIndexSave = 0;
 
     private index = new ChunkedIndexUpdateDateTuple();
     private askServerChunks: ChunkedIndexUpdateDateTuple[] = [];
@@ -145,7 +151,7 @@ export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
         private vortexStatusService: VortexStatusService,
         storageFactory: TupleStorageFactoryService,
         private tupleService: ChunkedIndexTupleService,
-        private deviceCacheControllerService: DeviceOfflineCacheControllerService
+        private deviceCacheControllerService: DeviceOfflineCacheService
     ) {
         super();
 
@@ -230,7 +236,7 @@ export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
 
         // If there is no offline support, or we're online
         if (
-            !this.deviceCacheControllerService.cachingEnabled ||
+            !this.deviceCacheControllerService.offlineModeEnabled ||
             this.vortexStatusService.snapshot.isOnline
         ) {
             let ts = new TupleSelector(ChunkedTuple.tupleName, {
@@ -257,14 +263,14 @@ export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
 
         // If we do have offline support
         if (this.isReady())
-            return this.getChunkedsWhenReady(modelSetKey, keys).then((docs) =>
+            return this.getChunksWhenReady(modelSetKey, keys).then((docs) =>
                 this._populateAndIndexObjectTypes(docs)
             );
 
         return this.isReadyObservable()
             .pipe(first())
             .toPromise()
-            .then(() => this.getChunkedsWhenReady(modelSetKey, keys))
+            .then(() => this.getChunksWhenReady(modelSetKey, keys))
             .then((docs) => this._populateAndIndexObjectTypes(docs));
     }
 
@@ -336,7 +342,7 @@ export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
             )
             .pipe(takeUntil(this.onDestroyEvent))
             .subscribe((payloadEnvelope: PayloadEnvelope) => {
-                this.processChunkedIndexsFromServer(payloadEnvelope);
+                this.processChunksFromServer(payloadEnvelope);
             });
 
         // If the vortex service comes back online, update the watch grids.
@@ -348,7 +354,7 @@ export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
 
     private areWeTalkingToTheServer(): boolean {
         return (
-            this.deviceCacheControllerService.cachingEnabled &&
+            this.deviceCacheControllerService.offlineModeEnabled &&
             this.vortexStatusService.snapshot.isOnline
         );
     }
@@ -432,96 +438,103 @@ export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
 
         if (this.askServerChunks.length == 0) return;
 
-        let indexChunk: ChunkedIndexUpdateDateTuple =
-            this.askServerChunks.pop();
-        let filt = extend({}, clientChunkedIndexWatchUpdateFromDeviceFilt);
-        filt[cacheAll] = true;
-        let pl = new Payload(filt, [indexChunk]);
-        this.vortexService.sendPayload(pl);
+        this.deviceCacheControllerService //
+            .waitForGarbageCollector()
+            .then(() => {
+                let indexChunk: ChunkedIndexUpdateDateTuple =
+                    this.askServerChunks.pop();
+                let filt = extend(
+                    {},
+                    clientChunkedIndexWatchUpdateFromDeviceFilt
+                );
+                filt[cacheAll] = true;
+                let pl = new Payload(filt, [indexChunk]);
+                this.vortexService.sendPayload(pl);
 
-        this._status.lastCheck = new Date();
-        this._notifyStatus();
+                this._status.lastCheck = new Date();
+                this._notifyStatus();
+            });
     }
 
     /** Process ChunkedIndexes From Server
      *
      * Process the grids the server has sent us.
      */
-    private processChunkedIndexsFromServer(payloadEnvelope: PayloadEnvelope) {
+    private async processChunksFromServer(
+        payloadEnvelope: PayloadEnvelope
+    ): Promise<void> {
         if (payloadEnvelope.result != null && payloadEnvelope.result != true) {
             console.log(`ERROR: ${payloadEnvelope.result}`);
             return;
         }
 
-        payloadEnvelope
-            .decodePayload()
-            .then((payload: Payload) => this.storeChunkedIndexPayload(payload))
-            .then(() => {
-                if (this.askServerChunks.length == 0) {
-                    this.index.initialLoadComplete = true;
-                    this._hasLoaded = true;
-                    this._hasLoadedSubject.next();
-                } else if (payloadEnvelope.filt[cacheAll] == true) {
-                    this.askServerForNextUpdateChunk();
-                }
-            })
-            .then(() => this._notifyStatus())
-            .catch(
-                (e) =>
-                    `ChunkedIndexCache.processChunkedIndexsFromServer failed: ${e}`
-            );
-    }
-
-    private storeChunkedIndexPayload(payload: Payload) {
-        let tuplesToSave: ChunkedIndexEncodedChunkTuple[] = <
+        const tuplesToSave: ChunkedIndexEncodedChunkTuple[] = <
             ChunkedIndexEncodedChunkTuple[]
-        >payload.tuples;
-        if (tuplesToSave.length == 0) return;
+        >payloadEnvelope.data;
 
-        // 2) Store the index
-        this.storeChunkedIndexChunkTuples(tuplesToSave)
-            .then(() => {
-                // 3) Store the update date
+        try {
+            await this.storeChunkTuples(tuplesToSave);
+        } catch (e) {
+            console.log(`PrivateSegmentLoaderService.storeChunkTuples: ${e}`);
+        }
 
-                for (let chunkedIndexIndex of tuplesToSave) {
-                    this.index.updateDateByChunkKey[
-                        chunkedIndexIndex.chunkKey
-                    ] = chunkedIndexIndex.lastUpdate;
-                }
+        if (this.askServerChunks.length == 0) {
+            this.index.initialLoadComplete = true;
+            await this.saveChunkCacheIndex(true);
+            this._hasLoaded = true;
+            this._hasLoadedSubject.next();
+        } else if (payloadEnvelope.filt[cacheAll] == true) {
+            this.askServerForNextUpdateChunk();
+        }
 
-                return this.storage.saveTuples(new UpdateDateTupleSelector(), [
-                    this.index,
-                ]);
-            })
-            .catch((e) =>
-                console.log(`ChunkedIndexCache.storeChunkedIndexPayload: ${e}`)
-            );
+        this._notifyStatus();
     }
 
     /** Store Index Bucket
      * Stores the index bucket in the local db.
      */
-    private storeChunkedIndexChunkTuples(
-        encodedChunkedIndexChunkTuples: ChunkedIndexEncodedChunkTuple[]
+    private async storeChunkTuples(
+        tuplesToSave: ChunkedIndexEncodedChunkTuple[]
     ): Promise<void> {
-        let retPromise: any;
-        retPromise = this.storage.transaction(true).then((tx) => {
-            let promises = [];
+        // noinspection BadExpressionStatementJS
+        const Selector = ChunkedIndexChunkTupleSelector;
 
-            for (let encodedChunkedIndexChunkTuple of encodedChunkedIndexChunkTuples) {
-                promises.push(
-                    tx.saveTuplesEncoded(
-                        new ChunkedIndexChunkTupleSelector(
-                            encodedChunkedIndexChunkTuple.chunkKey
-                        ),
-                        encodedChunkedIndexChunkTuple.encodedData
-                    )
-                );
-            }
+        if (tuplesToSave.length == 0) return;
 
-            return Promise.all(promises).then(() => tx.close());
-        });
-        return retPromise;
+        const batchStore: TupleStorageBatchSaveArguments[] = [];
+        for (const tuple of tuplesToSave) {
+            batchStore.push({
+                tupleSelector: new Selector(tuple.chunkKey),
+                vortexMsg: tuple.encodedData,
+            });
+        }
+
+        await this.storage.batchSaveTuplesEncoded(batchStore);
+
+        for (const tuple of tuplesToSave) {
+            this.index.updateDateByChunkKey[tuple.chunkKey] = tuple.lastUpdate;
+        }
+        await this.saveChunkCacheIndex(true);
+    }
+
+    /** Store Chunk Cache Index
+     *
+     * Updates our running tab of the update dates of the cached chunks
+     *
+     */
+    private async saveChunkCacheIndex(force = false): Promise<void> {
+        if (
+            this.chunksSavedSinceLastIndexSave <= this.SAVE_POINT_ITERATIONS &&
+            !force
+        ) {
+            return;
+        }
+
+        this.chunksSavedSinceLastIndexSave = 0;
+
+        await this.storage.saveTuples(new UpdateDateTupleSelector(), [
+            this.index,
+        ]);
     }
 
     /** Get ChunkedIndexs When Ready
@@ -529,7 +542,7 @@ export class ChunkedIndexLoaderService extends NgLifeCycleEvents {
      * Get the objects with matching keywords from the index..
      *
      */
-    private getChunkedsWhenReady(
+    private getChunksWhenReady(
         modelSetKey: string,
         keys: string[]
     ): Promise<ChunkedTuple[]> {
